function Au = Aut(d,t)
% Au = Aut(d,t) returns the auditory state at time t relative to the start of the
% simulation. This function utilizes information about transmission delays between the
% various components and generates the somatosensory state corresponding to the
% appropriate articulator configuration. The function calls on the representation
% specific function Ar2Au to determine the state. 

% Satrajit Ghosh
% (c) 2004 SpeechLab, Boston University

Au = zeros(d.sizeAu,1);

% Determine the motor command time slices corresponding to the sensor feedback
Mt = max(round(t-(d.tMAr+d.tArAu)),0);
P  = Pt(d,t-(d.tPM+d.tMAr+d.tArAu));

% Generate auditory state if time corresponds to after start of simulation (t > 0).  
if Mt & P,
    Au = Ar2Au(d.Ar(:,Mt),d);
end;

%% Earlier version
%Au = zeros(d.sizeAu,1);
%Mt = max(round(t-(d.tMAr+d.tArAu)),0);
%Pt = max(round(t-(d.tPM+d.tMAr+d.tArAu)),0);

%% check to ensure that something was articulated at an appropriate delay
%% prior to sensory activation
%if Pt,
%    Pt = d.P(:,Pt);
%end;
%if Mt & Pt,
%    Au = Ar2Au(d.Ar(:,Mt),d);
%end;