function S = St(d,t)
% S = St(d,t) returns the somatosensory state at time t relative to the start of the
% simulation. This function utilizes information about transmission delays between the
% various components and generates the somatosensory state corresponding to the
% appropriate articulator configuration. At this point in time, there is no difference
% between proprioceptive and tactile delays. The function calls on the representation
% specific function Ar2S to determine the state.

% Satrajit Ghosh
% (c) 2004 SpeechLab, Boston University

S = zeros(d.sizeS,1);

% Determine the motor command time slices corresponding to the sensor feedback
Mt = max(round(t-(d.tMAr+d.tArS)),0);

% Generate somatosensory state only if time corresponds to after start of simulation 
% (t > 0). 
if Mt,
    S = Ar2S(d.Ar(:,Mt),d);
end;


%% Earlier version
%  % Determine the motor and premotor command time slices corresponding to the
%  % sensor feedback
%  Mt = max(round(t-(d.tMAr+d.tArS)),0);
%  Pt = max(round(t-(d.tPM+d.tMAr+d.tArS)),0);
  
%  % check to ensure that something was articulated at an appropriate delay
%  % prior to sensory activation
%  if Pt,
%    Pt = d.P(:,Pt);
%  end;

%  % Only interested in sensory responses corresponding to production
%  if Mt & Pt,
%    %[Mt Pt]
%    S = Ar2S(d.Ar(:,Mt),d);
%  end;

