function d = create_diva(folder);
% D = CREATE_DIVA(FOLDER) creates the diva structure that holds the necessary data for
% running the simulations. In all functions including this one, the following
% nomenclature is used:
%    P  - Premotor
%    M  - Motor
%    Au - Auditory
%    S  - Somatosensory
%    Ar - Articulatory
%    Cb - Cerebellar
%    ff - feedforward
%    fb - feedback
%    dX - derivative of X

% Satrajit Ghosh
% (c) 2004 SpeechLab, Boston University

% Add path to representation specific folder
d.folder = folder;

d.simtime         = 0; % Duration of simulation
d.target_duration = 0; % Duration of target

d.P      =   []; % Premotor cell

d.dM     =   []; % Phasic motor cell
d.dMfbS  =   []; % Feedback command due to S  
d.dMfbAu =   []; % Feedback command due to A  
d.dMfb   =   []; % Combined feedback command
d.dMff   =   []; % Feedforward command
d.M      =   []; % Tonic motor cell
d.Ar     =   []; % Articulator configuration
d.Au     =   []; % Acoustic time vector
d.S      =   []; % Somatosensory time vector
d.DS     =   []; % Somatosensory error vector
d.DAu    =   []; % Auditory error vector
d.Cb     =   []; % Phasic motor cell

d.pff    =   1;    % Feedforward weight
d.pfb    =   0;    % Feedforward weight
d.pfbS   =   0;    % Feedback S weight
d.pfbAu  =   1;    % Feedback A weight
d.pCb    =   0;    % Feedback Cbm weight

d.ZPSlearn_rate = 1;  % Inverse command learning rate
d.ZPMlearn_rate = 1;  % Inverse command learning rate

% The spatiotemporal weight matrices
d.ZPM    =   [];  % Premotor to Motor weights [Motor target]
d.ZPAu   =   [];  % Premotor to Auditory Weights [Auditory target]
d.ZPS    =   [];  % Premotor to Somatosensory weights [Somatosensory target]

% The delays between various paths
d.tPM    =   0;
d.tMAr   =   42;
d.tArS   =   15;
d.tArAu  =   20;
d.tSM    =   3;
d.tAuM   =   3;
d.tPS    =   3;
d.tPAu   =   3;

d.tStart =  100;  % Simulation start time [movement towards initial state of target]
d.tEnd   =  100;  % Simulation end time  

% Vector sizes
d.sizeP = 0;
d.sizeM = 0;
d.sizeS = 0;
d.sizeAu= 0;
d.sizeAr= 0;

%Inertias. How much to weight the current time point with the previous timepoint. This
%is equivalent to a PD controller with P = 1 and D = -INERT. 
% a(t) = INERT*a(t-1) + (1-INERT)*a_hat(t)
%      = a_hat(t) - INERT*(a_hat(t)-a(t-1))
d.ZPM_INERT = 0.0;   % Weight learning intertia
d.M_INERT   = 0.8; % Tonic motor output inertia
d.FB_INERT  = 0.9; % Feedback inertia

d.plot_step  = 50; %plotting routine update in ms

% Add representation specific information to the structure. These functions should be
% in the representation specific folder
d = assign_sizes(d);
d = add_typespecificdata(d);