function [Y,data] = plotVT2(d,target,bM,cV,filename)
% PLOTVT(D,BM,CV,FILENAME) plots the movements of the vocal tract. bM is a
% boolean flag that specifies whether the entire movement (1) is to be
% plotted or just the part corresponding to the utterance (0). cV is
% another flag that determines whether a video should be generated. If set
% to 1, the 4th parameter filename must be provided.

% Satrajit Ghosh
% (c) SpeechLab, Boston University

if nargin<4,
    cV = 0;
end;
if cV,
    if nargin<5,
        error('No filename provided for AV');
    end;
end;

vt = d_opvt;    % create a vocal tract object
FPS = 60;       % Frames per second
tfs = 1;        % Time frame scaling value            

f = hanning(50);f = f/sum(f);
if ~isfield(d,'Maeda'),
    d.Maeda = d.Ar;
end
d.Maeda = filtfilt(f,1,d.Maeda')';

if bM,
    idx = 1:ceil(1000/FPS):length(d.M);
    for i=idx,
        % Set the voicing signal corresponding to the value of the
        % premotor cortical cell
        if i-d.tPM>0,
            AV = d.P(i-d.tPM);
        else,
            AV = 0;
        end;
        % plot vocal tract
        plot(vt,d.Maeda(:,i),AV);
        if cV,
            % capture frame
            mov(find(i==idx)) = getframe;
        end
    end;
else,
    % capture only utterance
    idx = 1:ceil(1000/FPS):length(d.ZPM);
    pt1 = find(d.P);pt1 = pt1(1)-1+d.tPM;
    for i=idx,
        % plot vocal tract
        plot(vt,d.Maeda(:,pt1+i),1);
        if cV,
            % capture frame
            mov(find(i==idx)) = getframe;
        end
    end;
end;

if bM,
    tf = length(d.M);
    %data.X = M2Ar(d.M(:,max(d.tPM,1):end),d);
    data.X = d.Maeda(:,max(d.tPM,1):end);
else,
    pt1 = find(d.P);pt1 = pt1-1+d.tPM;
    ptidx = pt1(1):pt1(end);
    tf = length(ptidx);
    data.X = d.Maeda(:,ptidx);
    %tf = length(d.ZPM);
    %data.X = M2Ar(d.ZPM,d);
end;

if bM,
    extradur = length(d.M)-length(d.ZPM);
else,
    extradur = 0;
end;
data.fs = 10000;
%tf = target.dur;
tf = length(d.ZPM);

if size(data.X,1)>size(d.Maeda,1),
    Art = M2Ar(d.M(:,d.tStart+[1:tf]),d);
    target.Ag0 = [[1:tf]',Art(8,:)',ones(tf,1)];target.Ag0(1,[1,3]) = 0;
    target.Ag0 = [[1:tf]',Art(9,:)',ones(tf,1)];target.AgP(1,[1,3]) = 0;
    target.F0 = [[1:tf]',Art(10,:)',ones(tf,1)];target.F0(1,[1,3]) = 0;
end
data.X = data.X(1:size(d.Maeda,1),:);

data.duration = target.dur+extradur;
if bM,
    bdur = find(d.P>0);
    bdur = bdur(1)-1;
    if bdur>0 & data.duration>(bdur+target.Ag0(end,1))
        data.Ag0 = [0 0 0;[bdur+target.Ag0(:,1),target.Ag0(:,2:3)];data.duration 0 1];
        data.AgP = [0 0.01 0;[bdur+target.AgP(:,1),target.AgP(:,2:3)];data.duration 0.01 1];
        data.F0  = [0 100 0;[bdur+target.F0(:,1),target.F0(:,2:3)];data.duration 100 1];
        data.AgP(2,3) = 1;
        data.Ag0(2,3) = 1;
        data.F0(2,3) = 1;
    elseif bdur>0
        data.Ag0 = [0 0.0 0;[bdur+target.Ag0(:,1),target.Ag0(:,2:3)]];
        data.AgP = [0 0.01 0;[bdur+target.AgP(:,1),target.AgP(:,2:3)]];
        data.F0  = [0 100 0;[bdur+target.F0(:,1),target.F0(:,2:3)]];
        data.AgP(2,3) = 1;
        data.Ag0(2,3) = 1;
        data.F0(2,3) = 1;
    elseif data.duration>(target.Ag0(end,1)),
        data.Ag0 = [target.Ag0;data.duration 0 1];
        data.AgP = [target.AgP;data.duration 0.01 1];
        data.F0  = [target.F0;data.duration 100 1];
    else,
        data.Ag0 = [target.Ag0];
        data.AgP = [target.AgP];
        data.F0  = [target.F0];
    end;
else,
    data.Ag0 = target.Ag0; %[0 0 1;tfs*tf 0 1];
    data.AgP = target.AgP; %[0 0.2 0;tfs*tf 0.2 1];
    data.F0  = target.F0; %[0 100 0;tfs*tf 100 1];
end;
% data.AgP
% data.Ag0

Y = synth1(vt,data);
% Interpolate data to match the scaled duration
if bM,
    Y = Y.*interp1(1:size(data.X,2),double(d.P([1:size(data.X,2)])>0),linspace(1,size(data.X,2),length(Y)))';
end;

% Scale the output to prevent clipping
Y = 0.99*Y./(max(abs(Y(:))));
%Y(1020:1045) = 0;
%Y(2891:2915) = 0;
%sound(Y,data.fs);


% Write out movie file and audio file if required
if cV,
    try
	movie2avi(mov,fullfile('results',[filename,'.avi']),'FPS',FPS, ...
		  'Compression','Indeo5');
    catch
	movie2avi(mov,fullfile('results',[filename,'.avi']),'FPS', ...
		  FPS);
    end
    %movie2avi(mov,fullfile('results',[filename,'.avi']),...
    %    'FPS',FPS,'Compression','MSVC','Quality',100);
    wavwrite(Y,data.fs,fullfile('results',[filename,'.wav']));
    
    % TODO: autocombine the output files and delete the individual files
    try
        combineWavVid(...
            fullfile('results',[filename,'.wav']),...
            fullfile('results',[filename,'.avi']),...
            fullfile('results',['comb_',filename,'.avi']));    
    catch
        disp('Combined video available only on windows machines');
    end
end;
