function d = sim_timestep(d,t)
% D = SIM_TIMESTEP(D,T) simulates a single time-point of the simulation. 

% Satrajit Ghosh
% (c) 2004 SpeechLab, Boston University

if t>1,
    d.M(:,t) = d.M(:,t-1); 
end;

% Get current target slice
P = Pt(d,t-d.tPM);
if (P>0) & (~d.started),
    d.started = 1;
end
if (P==0) & d.started,
    d.finished = 1;
end

d = update_sensory_states(d,t);
d = evaluate_errors(d,t);

% phasic commands
d.dMfbS(:,t)  = ZSMt(d,t);
d.dMfbAu(:,t) = ZAuMt(d,t);
d.dMff(:,t)   = ZPMt(d,t);
d.dMfb(:,t)   = (d.pfbS*d.dMfbS(:,t)+d.pfbAu*d.dMfbAu(:,t)); 
%/(d.pfbS*sum(abs(d.dMfbS(:,t)))+d.pfbAu*sum(abs(d.dMfbAu(:,t))));
if (P>1),
    d.dMfb(:,t) = d.FB_INERT*d.dMfb(:,t-1) + (1-d.FB_INERT)*d.dMfb(:,t);
end
d.Cb(:,t)     = Cbmt(d,t);
d.dM(:,t)     = (d.pff*d.dMff(:,t)+d.pfb*d.dMfb(:,t)+d.pCb*d.Cb(:,t)); %/(d.pfb+d.pff);

% tonic command
d.M(:,t)       = d.M(:,t) + d.dM(:,t);
d.M(:,t)       = min(max(d.M(:,t),0),1);
d.M(2:2:end,t) = 1-d.M(1:2:end,t);

%if (pt>1) %& (norm(d.dMfb(:,t))>1e-2),
%if (t>1),
if t>1,
    d.M(:,t)  = d.M_INERT*d.M(:,t-1)+(1-d.M_INERT)*d.M(:,t);
end
d.Ar(:,t)     = M2Ar(d.M(:,t),d);

if d.pert,
    d = Pert(d,t);
end
