% Create the DIVA structure
addpath(fullfile(pwd,'DIVA'));
d = diva('formants2');

% Load and setup the target
target_str = 'good_doggie';
target = load(['targets',filesep,'synth_targets'],['target_',target_str]);
target = target.(char(fieldnames(target)));
d = setup_target(d,target);

% Set learning parameters
d.pff  = 1;
d.pfbAu= 1;
d.pfbS = 0;
d.pfb  = 0;
d.ZPSlearn_rate = 1;
d.ZPMlearn_rate = 1;
d.FB_INERT      = 0.9;
d.ZPM_INERT     = 0.0;
d.M_INERT       = 0.8;

%Initialize and run simulation
d.started = 0;
d.finished= 0;
d_learn(1) = d; N = 10;
target1 = target;
tmean = target.f.mean;
range = 1:d.target_duration;
for i=2:N,
    % Create diminishing target region as learning progresses
    target1.f.var(:,1) = (1-exp(i-N))*2500;
    target1.f.var(:,2) = (1-exp(i-N))*10000;
    target1.f.var(:,3) = (1-exp(i-N))*10000;
    tvar  = target1.f.var;
    d_learn(i-1).ZPAu    = max([tmean(range,:)-sqrt(tvar(range,:)),tmean(range,:)+sqrt(tvar(range,:))]',50);
    
    % Increase somatosensory feedback contribution as learning progresses
    d_learn(i-1).pfbS          = exp(i-N);
    d_learn(i-1).pfb           = exp(i-N);
    d_learn(i-1).ZPSlearn_rate = 1-exp(i-N);

    d_learn(i)=simulate(d_learn(i-1));
end;

% Produce the learned target

d = d_learn(end);

% Switch target back to full range
target1.f.var(:,1) = 2500;
target1.f.var(:,2) = 10000;
target1.f.var(:,3) = 10000;
tvar   = target1.f.var;
d.ZPAu = max([tmean(range,:)-sqrt(tvar(range,:)),tmean(range,:)+sqrt(tvar(range,:))]',50);

simulate(d);

d.Maeda = Ar2Maeda(d.Ar,d);
[Y,data] = generate_video(d,target,target_str);
save(['synth_target_',target_str],'d','target');
soundview(Y,10000);
