function dMfbS = ZSMt(d,t)
% This function generates corrective movements corresponding to the
% perceived somatosensory error. Since this function can only render its
% effect after a certain time lapse, it will not be useful for ballistic
% movements.

% Satrajit Ghosh
% (c) SpeechLab, Boston University

persistent DeltaSold DeltaSum

dMfbS   = zeros(d.sizeM,1);

% Initialize dummy storage variable to hold previous time-step
if isempty(DeltaSold),
    DeltaSold = zeros(d.sizeAr,1); %M2Ar(dMfbS,d);
end

% Determine the error in somatosensory cortex
st = max(t-d.tSM,0);
if st,
    DeltaS = d.DS(:,st); %DeltaAut(d,t-d.tAuM);
else,
    return;
end

% The 'mt' index serves as an index into the motor command that caused the
% error.
% The 'pt' index looks at the premotor cells to see if something was being
% articulated. 
pt  = Pt(d,t-(d.tPM+d.tMAr+d.tArS+d.tSM));

% Generate corrective movements for that error
if pt,
    mt  = max(t-(d.tMAr+d.tArS+d.tSM),0);
    
    % DeltaS
    % DeltaS = DeltaS(1:d.sizeAr) + sum(DeltaS((d.sizeAr+1):end))*d.WSM*[d.S(1:d.sizeAr,t-d.tSM);DeltaS((d.sizeAr+1):end)];
    % DeltaS = pseudoinv01(M2Ar(d.M(:,mt),d),DeltaS(1:d.sizeAr),@Ar2Proprioceptive,[d],d.iparams);
    d.iparams.DELTA = 5;
    DeltaS = DeltaS(1:d.sizeAr) + pseudoinv01_S(M2Ar(d.M(:,mt),d),DeltaS((d.sizeAr+1):end),@Ar2Tactile,[d],d.iparams);
    DeltaSold = d.DS_INERT*DeltaSold + (1-d.DS_INERT)*DeltaS; % + 0.001*DeltaSum;
    
    dMfbS = Ar2M(DeltaSold,d)-0.5;
    %dMfbS = Ar2M(DeltaS(1:7) + d.WSM*[zeros(7,1);DeltaS(8:15)],d)-0.5;
    %fprintf(['DeltaS: ',num2str(mean(abs(DeltaS))),' \n']);
end;