function [p_pos,tongue_xy] = pellet_positions(Ar,N)
% [P_POS,TONGUE_XY] = PELLET_POSITIONS(AR,N) returns the xy coordinates of each Maeda
% saggital segment [P_POS] along with equidistant xy coordinates along the midsaggital
% tongue surface. N specifies the total number of points along the tongue surface. N
% defaults to 500.

% Satrajit Ghosh
% (c) 2004 SpeechLab, Boston University

vt = d_opvt;
p_pos = zeros(4,29,length(Ar));
for n0=1:length(Ar),
    [F,B,A,Af,P1] = convert1(vt,Ar(:,n0));
    p_pos(:,:,n0) = P1{:};
end

x = squeeze(p_pos(1,6:27,:)); y = squeeze(p_pos(2,6:27,:));
d = cumsum(sqrt((y(2:end,:)-y(1:end-1,:)).^2 + (x(2:end,:)-x(1:end-1,:)).^2));

if nargin<2 | isempty(N),
  N = 500;
end

tongue_xy = zeros(N,2,size(d,2));
for n0=1:size(d,2),
    tongue_xy(:,:,n0) = interp1([0;d(:,n0)],[x(:,n0),y(:,n0)],linspace(0,d(end,n0),N));
end